% function Y = nscore_main(seqfile);
% Calculate N-score from input sequences
% Input: 
% seqfile --- a text file containing input sequences.
% Note: Each input sequence must be 131 bp long. 
% For longer sequences, the first 131 bps are extracted. 
% Shorter sequences will lead to runtime error. 
% sasres --- an externally provided structure containing the linear
% regression coefficients
% sasres.theta --- the intercept
% sasres.beta  --- the linear coefficients.
%
% Output: 
% Y --- a structure containing the sequence and resulting N-scores.
% Y.seq --- Input sequences
% Y.nscore --- N-scores.
%
% Example:
% load('select_var_WT_L2vb.mat');
% Y = nscore_main('sample_sequence.txt', sasres);

function Y = nscore_main(seqfile, sasres);

clear Y Z;

fid = fopen(seqfile);
seq = textscan(fid, '%s');
Z.seq = seq{1};
fclose(fid);

nseq = length(Z.seq);

dicode = [11 12 13 14 21 22 23 24 31 32 33 34 41 42 43 44];

%derive complementary sequences
for i = 1:nseq,
    Z.seq{nseq+i} = seq_complement(Z.seq{i});
end

npos_total = 131;
npos_good = 128;
pos_good = 2:129;

Z.code = zeros(nseq, npos_total);
Z.dicode = zeros(nseq, npos_total-1);
for i = 1:2*nseq,
    Z.code(i, :) = seq2code(Z.seq{i}, npos_total);
    Z.dicode(i, :) = Z.code(i, 1:end-1)*10 + Z.code(i, 2:end);
end

%count dinucleotide distribution
Z.dicode_freq = zeros(nseq, npos_good, 16);
for ns = 1:nseq,
    for i = 1:16,    
        dicode_freq_forward = (Z.dicode(ns, [pos_good(1)-1 pos_good pos_good(end)+1]) == dicode(i)) + 0; %average over 3bp
        dicode_freq_reverse = (Z.dicode(ns+nseq, [pos_good(1)-1 pos_good pos_good(end)+1]) == dicode(i)) + 0; %also count reverse seqeunce
        dicode_freq_tmp = 0.5*(dicode_freq_forward + dicode_freq_reverse);
        dicode_freq_tmp = ( dicode_freq_tmp(1:end-2) + dicode_freq_tmp(2:end-1)  + dicode_freq_tmp(3:end) )/3;
        Z.dicode_freq(ns, :, i) = dicode_freq_tmp;  
    end
end

%wavelet transform
nlev = 7;
[s, cL] = wavedec(Z.dicode_freq(1, :, 1), nlev, 'haar');
lwt = length(s);
Z.dwt = zeros(nseq, lwt, 16);
for ns = 1:nseq,
    for i = 1:16,
        [dwt_tmp, Z.wtlev] = wavedec(Z.dicode_freq(ns, :, i), nlev, 'haar');
        Z.dwt(ns, :, i) = dwt_tmp;
    end
end

we = PatternSymWTL2vb(Z.dwt, Z.wtlev);

nscore = zeros(nseq, 1);
for k = 1:length(sasres.iselect),
    nscore = nscore + we(:, sasres.iselect(k), sasres.jselect(k))*sasres.beta(k);
end
nscore = nscore + sasres.theta;

Y.seq = Z.seq(1:nseq);
Y.nscore = nscore;

end

%--------------------------------------------------------------------
function seq2 = seq_complement(seq1);

seq2 = seq1;
seq2(:) = '-';
I = find(seq1 == 'A');
seq2(end+1-I) = 'T';
I = find(seq1 == 'C');
seq2(end+1-I) = 'G';
I = find(seq1 == 'G');
seq2(end+1-I) = 'C';
I = find(seq1 == 'T');
seq2(end+1-I) = 'A';

%check if there are unresolved characters
J = find(seq2(:) == '-');
if ~isempty(J),
    error('Error: Unresolved characters found!');
end

end

%----------------------------------------------------------------
function code = seq2code(str, n);

code = zeros(size(str)) + NaN;
I = find(str == 'A');
code(I) = 1;
I = find(str == 'C');
code(I) = 2;
I = find(str == 'G');
code(I) = 3;
I = find(str == 'T');
code(I) = 4;

code = code(1:n);

end


%---------------------------------------------------
function pattern = PatternSymWTL2vb(wt, lev);

nwt = size(wt, 1);
npos = size(wt, 2);
nlev = length(lev)-1;

pattern = zeros(nwt, nlev, 16);
for i = 1:nwt,
    for j = 1:16,
        C = reshape(wt(i, :, j), 1, npos);
        ec = C*C';
        if norm(C) > 0,
            [ea, ed] = wenergy(C, lev);
            ea = ea*ec/100;
            ed = ed*ec/100;
        else,
            ea = 0;
            ed = zeros(1, nlev-1);
        end  
        pattern(i, :, j) = [ea ed(end:-1:1)]; 
    end
end

end